#!/bin/perl -w
#################################################################################
# Created       : Martin Foster
# Modified      : 01/14/2000
#################################################################################
#
# User Configuration -	Script part of Webchat designed to modify user information 
# 			rights.  Contains comprehensive search parameter.
# Copyright (C) 2000  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : mfoster@redwhite.com
#       Address : Suite 209 - 355 5th Ave NE
#                 Calgary, Alberta
#                 T2R 0K9
#
#################################################################################
use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Webchat::Database;						# Database handler
use Webchat::Dbm;						# DBM Addition
use Webchat::Login;						# Login functionality
use Webchat::Param;						# Parameter control
use Webchat::Table;						# Table control


#################################################################################
# Data Members
#################################################################################
my $Database;							# Database handle
my $cgi;							# Common gateway interface handle

my %config;							# System hash
my %param;							# Parameter hash


#################################################################################
# Program Area
#################################################################################

	$cgi = new CGI;						# Create CGI Handle

	Webchat::Database::DatabaseConnect(\$Database);		# Connect to MySQL
	Webchat::Param::GetParam($cgi, \%param);		# Retreive parameters

	print $cgi->header();					# File header
	print $cgi->start_html('User Administration');		# HTML header			

	# Verification of login before admin screens
	if (Webchat::Login::GetVerificationAdmin($Database, $cgi, \%param))
	{
		print $cgi->h1('User Administration'), "\n";	# Document title

		# Basic search prompt
		if ((!defined($param{'RANGE'})) && (!defined($param{'FIELD'})) && (!defined($param{'LOGIN'})))
		{ 
			UserSearch($Database, $cgi, \%param);
		}

		# Basic search with user selection and removal
		elsif (!defined($param{'LOGIN'}))
		{
			UserSelect($Database, $cgi, \%param);
			UserSearch($Database, $cgi, \%param);
		}

		# Handles the primary modifications of a user
		elsif (defined($param{'LOGIN'}))
		{
			UserModify($Database, $cgi, \%param);
			UserSearch($Database, $cgi, \%param);
		}
	}
	print $cgi->end_html();					# HTML footer
	$Database->disconnect();				# Disconnect from database


#################################################################################
# Sub-Routines
#################################################################################

#####################
# SQL Handler
# 
# Looks at parameters and returns a formatted SQL statement to be used for searches

sub SqlHandler							# Parameter Hash
{
	#####################
	# Data members
	my $param = shift;					# Parameter hash
	my $sql;						# SQL query	

	my @range = qw(Administrators Supervisors Non-Priviledged All-Encompasing);
	my @field = qw(E-Mail Login Name);


	#####################
	# Program area
	$sql = 'SELECT PuppeteerLogin, PuppeteerName, PuppeteerEMail FROM Puppeteer';

	# Build SQL query

	# Admin privs
	if ($param->{'RANGE'} eq $range[0])
	{ $sql = "$sql WHERE PuppeteerAdminPrivs='yes'"; }

	# Supervisor privs
	elsif ($param->{'RANGE'} eq $range[1])
	{ $sql = "$sql WHERE PuppeteerSuperPrivs='yes'"; }

	# Non-Priviledged users
	elsif ($param->{'RANGE'} eq $range[2])
	{ $sql = "$sql WHERE PuppeteerSuperPrivs='no' AND PuppeteerAdminPrivs='no'"; }
	

	# Only complete following if nothing (or near to) in keywords
	unless (($param->{'KEYWORD'} =~ tr/A-Za-z1-9@//) < 3)
	{
		# Addition of AND if WHERE is already applied
		unless ($param->{'RANGE'} eq  $range[3])
		{ $sql = "$sql AND"; }
		else
		{ $sql = "$sql WHERE"; }
		

		# Field selection
		# E-Mail field search
		if ($param->{'FIELD'} eq $field[0])
		{ $sql = "$sql PuppeteerEMail"; }

		# Login name search
		elsif ($param->{'FIELD'} eq $field[1])
		{ $sql = "$sql PuppeteerLogin"; }

		# Full name search
		elsif ($param->{'FIELD'} eq $field[2])
		{ $sql = "$sql PuppeteerName"; }

		# Completion of SQL statement
		$sql = "$sql LIKE '\%$param->{'KEYWORD'}\%'";
	}

	return "$sql ORDER BY PuppeteerLogin";			# Return statement to parent
}


#####################
# User Modify
#
# Will either prompt for information or make changes and list the changed 
# values to the user.   Only one value can be changed at a time.  

sub UserModify							# Database, CGI, Parameter reference
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $Statement;						# Statement handle

	my $cgi = shift;					# CGI handle
	my $param = shift;					# Parameter reference
	my $radioadmin;						# Radio adminstration
	my $radiosuper;						# Radio supervisory 

	my $login;						# Login name

	my @values;						# Array of 
	my @choices = qw(yes no);				# Yes or no


	#####################
	# Program area

	# Change username to normal
	$login = $param->{'LOGIN'};
	$login =~ s/_/ /g;

	# If save then save
	if (defined($param->{'SAVE'}))				# Saved parameters
	{
		# Make changes
		$Database->do("UPDATE Puppeteer SET PuppeteerPassword=?,  PuppeteerName=?,
				PuppeteerEMail=?, PuppeteerAdminPrivs=?, PuppeteerSuperPrivs=?
				WHERE PuppeteerLogin=?", 
				{}, $param->{'PASSWORD'}, $param->{'FULLNAME'}, 
				$param->{'EMAIL'}, $param->{'ADMIN'}, 
				$param->{'SUPER'}, $login );
	}

	# Data display
	# Prepare and execute
	$Statement = $Database->prepare("SELECT * FROM Puppeteer WHERE PuppeteerLogin=?");
	$Statement->execute($login);

	@values = $Statement->fetchrow();
	$Statement->finish();

	print $cgi->start_form(), "\n";				# Start of form
	Webchat::Table::MakeTop($cgi);				# Table top

	# Login, Password, Name and EMail
	Webchat::Table::MakeValid($cgi, '<B>Login Name</B>', "<B>$values[0]</B>");
	Webchat::Table::MakeValid($cgi, '<B>Password</B>',  $cgi->textfield('PASSWORD', $values[1], 25, 13));
	Webchat::Table::MakeValid($cgi, '<B>Full Name</B>', $cgi->textfield('FULLNAME', $values[2], 25, 30));
	Webchat::Table::MakeValid($cgi, '<B>E-Mail</B>',    $cgi->textfield('EMAIL', $values[3], 25, 45));
	Webchat::Table::MakeBlank($cgi);

	# Timestamp, Formatting, Chat Prefs and Javascript (cannot be changed)
	Webchat::Table::MakeValid($cgi, '<B>Timestamp </B>', "<I>$values[4]</I>");
	Webchat::Table::MakeValid($cgi, '<B>Formatting</B>', "<I>$values[5]</I>");
	Webchat::Table::MakeValid($cgi, '<B>Chat Prefs</B>', "<I>$values[6]</I>");
	Webchat::Table::MakeValid($cgi, '<B>Javascript</B>', "<I>$values[7]</I>");
	Webchat::Table::MakeBlank($cgi);

	# Preperation of radio button labels
	$radioadmin = $cgi->radio_group('ADMIN', \@choices, $values[8]);
	$radiosuper = $cgi->radio_group('SUPER', \@choices, $values[9]);

	# Adminstrative and Supervisory controls
	Webchat::Table::MakeValid($cgi, '<B>Administrative</B>', $radioadmin);
	Webchat::Table::MakeValid($cgi, '<B>Supervisory   </B>', $radiosuper);
	Webchat::Table::MakeBlank($cgi);

	# Submit and Reset
	Webchat::Table::MakeValid($cgi, $cgi->submit('Change'), $cgi->reset('Reset'));
	Webchat::Table::MakeBottom($cgi);

	# Embed login information
	Webchat::Param::EmbedNormal($Database, $cgi, $param);
	print "\n", $cgi->hidden('LOGIN', $param->{'LOGIN'}), "\n";
	print "\n", $cgi->hidden('SAVE', 'Aye'), "\n";

	print $cgi->end_form(), "\n";				# End of form
	print "</TABLE>\n";					# End table/row
		
}


#####################
# User Search
#
# Allows one to select range of users to select, Admin, Supervisors, et cetera

sub UserSearch							# Database, CGI
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $cgi = shift;					# CGI handle
	my $param = shift;					# Parameter list

	# Range selection label 
	# Must not be changed
	my @range = qw(Administrators Supervisors Non-Priviledged All-Encompasing);
	my @field = qw(E-Mail Login Name);
	my $radiorange;						# Range radio button
	my $radiofield;						# Field ratio button


	#####################
	# Program area
		
	print $cgi->hr(), "\n";					# Horizontal row
	print $cgi->start_form();				# Begnning HTML form

	$radiorange = $cgi->radio_group('RANGE', \@range, $range[3], 'true');
	$radiofield = $cgi->radio_group('FIELD', \@field, $field[0], 'true');

	Webchat::Table::MakeTop($cgi);				# Table Top

	# Search Range
	Webchat::Table::MakeValid($cgi, '<B>Search Range</B>', $radiorange);
	Webchat::Table::MakeBlank($cgi);			# Spacer

	# Field and keyword
	Webchat::Table::MakeValid($cgi, '<B>Search Field</B>', $radiofield);
	Webchat::Table::MakeValid($cgi, '<B>Search Words</B>', $cgi->textfield('KEYWORD', '', 25, 25));	
	Webchat::Table::MakeBlank($cgi);			# Spacer	

	# Submit and bottom
	Webchat::Table::MakeValid($cgi, $cgi->submit('Search'), ''); 
	Webchat::Table::MakeBottom($cgi);

	# Embed login information
	Webchat::Param::EmbedNormal($Database, $cgi, $param);

	print $cgi->end_form(), "\n";				# Ending HTML form
	print $cgi->hr(), "\n";					# Horizontal row
}


#####################
# User Select
#
# Displays initial query and allows to select individual user

sub UserSelect							# Database, CGI, Parameter List
{
	#####################
	# Data members
	my $Database = shift;					# Database handle
	my $Statement;						# Statement shift
	my $query = SqlHandler(\%param);			# SQL Query

	my $cgi = shift;					# CGI handle
	my $param = shift;					# Parameter list reference

	my $color = 0;						# Colour changer

	my @values;						# Received values


	#####################
	# Program area	
	
	# If defined, remove	
	if (defined($param->{'DELETE'}))
	{
		# Return to orginal condition
		my $login = Webchat::Login::Unmangle($param->{'DELETE'});

		# Remove Puppets
		$Statement = $Database->prepare("SELECT PuppetName FROM Puppet WHERE PuppeteerLogin=?");
		$Statement->execute($login);

		# Clense database of puppets
		while ($_ = $Statement->fetchrow())
		{
			$Database->do("DELETE FROM PuppetInvitation WHERE PuppetName=?", {}, $_);
			$Database->do("DELETE FROM PuppetIgnore WHERE PuppetName=?", {}, $_);
		}

		# Remove puppeteer from tables
		$Database->do("DELETE FROM RealmSupervisor WHERE PuppeteerLogin=?", {}, $login);
		$Database->do("DELETE FROM PuppetIgnore WHERE PuppeteerLogin=?", {}, $login);
		$Database->do("DELETE FROM PuppeteerRegistration WHERE PuppeteerLogin=?", {}, $login);
		$Database->do("DELETE FROM PuppeteerRestriction WHERE PuppeteerLogin=?", {}, $login);
		$Database->do("DELETE FROM Puppet WHERE PuppeteerLogin=?", {}, $login);
		$Database->do("DELETE FROM Puppeteer WHERE PuppeteerLogin=?", {}, $login);
	}

	# Data gathering and display
	# Prepare and execute
	$Statement = $Database->prepare($query);
	$Statement->execute();
	
	print "<TABLE WIDTH=\"100%\">";

	# Display header
	print "\t<TR>\n";
	print "\t\t<TD><B>Login Name</B></TD>\n";
	print "\t\t<TD><B>Full Name</B></TD>\n";
	print "\t\t<TD><B>E-Mail address</B></TD>\n";
	print "\t\t<TD></TD>\n";
	print "\t\t<TD></TD>\n";
	print "\t</TR>\n";


	# Fetch rows of data
	while (@values = $Statement->fetchrow())
	{
		# Determin colour of row
		if ($color != 0)
		{ print "\t<TR>\n"; $color = 0; }
		else
		{ print "\t<TR BGCOLOR=\"\#CCCCCC\">\n";  $color = 1; }

		print "\t\t<TD><B>$values[0]</B></TD>\n";
		print "\t\t<TD>$values[1]</TD>\n";
		print "\t\t<TD><A HREF=\"mailto:$values[2]\">$values[2]</A></TD>\n";

		# Remove spaces
		$values[0] =~ s/\s/_/g;

		print "\t\t", $cgi->start_form();		# Start form
		print "\t\t\t", $cgi->hidden('LOGIN', $values[0]), "\n";
		Webchat::Param::EmbedNormal($Database, $cgi, $param);
		print "\t\t<TD>\n";
		print "\t\t\t", $cgi->submit('Modify'), "\n";
		print "\t\t</TD>\n";
		print "\t\t", $cgi->end_form(), "\n";		# End form

		print "\t\t", $cgi->start_form();		# Start form
		print "\t\t\t", $cgi->hidden('RANGE',   $param->{'RANGE'}), "\n";
		print "\t\t\t", $cgi->hidden('FIELD',   $param->{'FIELD'}), "\n";
		print "\t\t\t", $cgi->hidden('KEYWORD', $param->{'KEYWORD'}), "\n";
		print "\t\t\t", $cgi->hidden('DELETE',  $values[0]), "\n";
		Webchat::Param::EmbedNormal($Database, $cgi, $param);

		print "\t\t<TD>\n";
		print "\t\t\t", $cgi->submit('Remove'), "\n";
		print "\t\t</TD>\n";
		print "\t\t", $cgi->end_form(), "\n";		# End form

		print "\t</TR>\n";				# End row
	}

	$Statement->finish();					# Finish SQL statement
	print "</TABLE>\n";					# End of table
}
